.text
.globl  factorial
.type   factorial, @function

factorial:
    pushl %ebp
    # Put first parameter in %ecx
    movl 8(%esp), %ecx
    # Put second parameter in %ebx
    movl 12(%esp), %ebx

    # Call the actual code
    jmp factorial_start

# Return from the function
function_return:
    pop %ebp
    ret

################################################
# THIS IS THE ACTUAL DEFINITION HERE
#
# factorial procedure
# parameter: An integer to perform factorial on
#            in %ecx
# parameter: A pointer to an unsigned long long
#            for the result
#            in %ebx
# return: Whether or not there was overflow
#         in %eax
#
################################################


factorial_start:
    movl    %ecx, %eax      # Move the param into %eax (so we can use the next inst)
    cltd                    # Convert eax to quad word in %eax and %edx
    
    cmpl    $1, %eax        # Test if input is exactly 1
    jz      no_overflow     # if so , return 1
    

loop:
    decl    %ecx            # Decrement the param (multiplying by i-1)

    mull    %ecx            # Multiply by the decremented value
    jb      overflow        # Since CF is valid, there was an overflow

    cmpl    $1, %ecx        # Test if %ecx is now exactly 1
    jz      no_overflow     # We reached 1 without overflow. Set up the return value and return
    jmp     loop            # Continue the loop

no_overflow:
    # Note: At this point, we have the result and it's not an overflow
    # However, the result is in 2 registers (%eax:%edx).
    movl    %eax, (%ebx)    # Move the low-order 32 bits to where they should be in memory
    movl    %edx, 4(%ebx)   # Move the high-order bits to where they should be in memory
    xorl    %eax, %eax      # Put 0 in the return value
    jmp     function_return

overflow:
    # We had an overflow. However, the result is still valid since we used mull op
    # However, the result is in 2 registers (%eax:%edx), so it needs to be moved.
    movl    %eax, (%ebx)    # Move the low-order 32 bits to where they should be in memory
    movl    %edx, 4(%ebx)   # Move the high-order bits to where they should be in memory
    movl    $1, %eax        # There was an overflow, so set the return value to true
    jmp     function_return
